# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import pango
from gobject import PARAM_READWRITE, TYPE_PYOBJECT
from Parser import Smiley
from emesenelib.common import escape

class SmileyRenderer(gtk.GenericCellRenderer):
    '''Cellrenderer with smiley support. Based on many examples from
    code.google.com, because there isn't any documentation.'''

    __gproperties__ = {
        'markup':
            (TYPE_PYOBJECT,\
             'Smiley markup',\
             'A list with strs, Smileys, and \ns',
             PARAM_READWRITE),
        'ellipsize': # alpha
            (int, 'PangoEllipsizeMode', '', 0, 3, 0, PARAM_READWRITE),
        'wrap': # alpha
            (int, 'PangoWrapMode', '', -1, 2, -1, PARAM_READWRITE),
        'fixed':
            (bool, 'Fixed width', '', False, PARAM_READWRITE),
        'widget_width':
            (int, \
            'Widget Width',
            "A tip on widget's width for wrap and ellipsize operations",
            0, 2**16, 0, # TODO: remove that max size
            PARAM_READWRITE)
    }

    def __init__( self, cache = {} ):
        self.__gobject_init__()

        self.debug_string = "[DEBUG SMILEYS]" # contacts with this string are debugged
        self.smiley_margin = 2
        self.xpad = 2
        self.ypad = 1
        
        self.lastTextHeight = 0
        self.cache = cache

        self.ellipsize = 0
        self.wrap = -1
        self.fixed = False
        self.markup = []
        self.widget_width = 0

        self.cellwidth = 0
        self.cellheight = 0

    def canDebug(self):
        '''Return True if the user has debug_string in the nickname'''
        if str(self.markup).find(self.debug_string) != -1:
            return True
        return False
        
    def do_get_property(self, prop):
        return getattr(self, prop.name)

    def do_set_property(self, prop, value):
        setattr(self, prop.name, value)

    def on_render(self, win, widget, background_area, cell_area, expose_area, flags):
        '''Called by gtk to render the cell.'''

        self.renderPath('render', cell_area, widget, win, \
                        background_area, expose_area, flags)

    def on_get_size(self, widget, cell_area):
        '''Returns the size of the cellrenderer'''

        x_offset = y_offset = 0
        width, height = self.renderPath('size', cell_area, widget)
        
        if self.canDebug():
            print "get size:", x_offset, y_offset, width, height

        if self.fixed:
            return x_offset, y_offset, width, height
        else:
            # workaround for ellipsize
            return x_offset, y_offset, 0, height
        
    def renderPath(self, mode, cell_area, widget, win=None, background_area=None, expose_area=None, flags=None):
        '''Draw and/or get the size, depending of the mode.
        Calculates positions based on width and height'''
        if mode not in ['render','size']: return
        render = (mode=='render')

        start_xy = (0,0)
        if render:
            start_xy = (cell_area.x + self.xpad, cell_area.y + self.ypad)
        else:
            class fakecell:
                x = 0
                y = 0
                width = 2**8
                height = 0
            cell_area = fakecell()
            if self.widget_width:
                cell_area.width = self.widget_width

        # start_x is used in the for line in lines
        y = start_xy[1]  # start
        size_x = highersize_x = 0  # per-item
        size_y = highersize_y = 0  # per-line
        total_y = 0  # end
        if type(self.markup) in (str, unicode):
            data = [self.markup]
        else:
            # i don't know why i have to copy this, but it works
            data = [x for x in self.markup]

        item = [''] # a type that holds *any* other variable safely
        def spop( list, n=0 ):
            '''safe pop'''
            try:
                item[0] = list.pop(n)
                return True
            except Exception, e:
                return False
        
        while data:
            # init line
            x = start_xy[0]
            y += highersize_y
            size_y = highersize_y = 0
            while spop(data) and item[0] != '\n':
                # draw the block and/or get the size
                size_x, size_y = self.draw(not render, item[0], x,y, win, \
                                           widget, cell_area, flags)
                # add the width now, save the height for the next line
                x += size_x
                if size_y > highersize_y:
                    highersize_y = size_y
                
            # add the total height, and set highersize_x if gt
            total_y += highersize_y
            if (x-start_xy[0]) > highersize_x:
                highersize_x = x-start_xy[0]
            
        width, height = highersize_x, (total_y + self.ypad)
        return width,height
        
    def draw(self, dummy, item, x,y, win, widget, cell_area, flags):
        '''minimal abstraction between pixbuf and text
        - If item is an animation, get pixbuf
        - If item is a pixbuf, call win.draw_pixbuf
        - If item is a string, call paint_layout
        Then save start and end points. Returns size_xy'''

        if type(item) == Smiley:
            if self.lastTextHeight == 0:
                tempLayout = widget.create_pango_layout("")
                self.lastTextHeight = tempLayout.get_pixel_size()[1]

            SM = self.smiley_margin
            size = self.lastTextHeight + SM
            item.scale(self.cache, size)
            if not dummy:
                win.draw_pixbuf(None, item.pixbuf, 0, 0, int(x), int(y-SM), -1, -1)
            size_xy = (item.pixbuf.get_width() + SM, item.pixbuf.get_height())
        elif type(item) in (str, unicode):
            layout = widget.create_pango_layout("")
            layout.set_markup(item)
            layout.set_indent(x)
            #layout.set_width((cell_area.width-60) * pango.SCALE)
            
            #if self.ellipsize == pango.ELLIPSIZE_END:
            #    layout.set_width((cell_area.width-20) * pango.SCALE)
            #    layout.set_single_paragraph_mode(True)

            #if self.wrap != -1:
            #    layout.set_wrap(self.wrap)
            #    layout.set_single_paragraph_mode(False)
                
            size_xy = layout.get_pixel_size()
            self.lastTextHeight = size_xy[1]  # for smileys
            if not dummy:
                gc = widget.style.fg_gc[flags]
                win.draw_layout(gc, x, y, layout)
        else:
            print "wtf?", type(item), repr(item)
        return size_xy

class SmileyLabel( gtk.CellView ):
    def __init__( self, text=[] ):
        gtk.CellView.__init__( self )
        self.cache = {}
        self.renderer = SmileyRenderer( self.cache )
        self.renderer.set_property('markup', text)
        self.pack_start(self.renderer)

    def set_text( self, text=[]):
        self.renderer.set_property('markup', text)
        
    def set_markup( self, text=[] ):
    	self.set_text( text )
    
    def set_line_wrap( self, wrap ):
        if wrap:
            mode = 0
        else:
            mode = -1
        self.renderer.set_property( 'wrap', mode )
        
    def set_fixed( self, fixed ):
        self.renderer.set_property( 'fixed', fixed )

    def set_width( self, width ):
        self.renderer.set_property( 'widget_width', width )

    def set_ellipsize( self, value ):
        self.renderer.set_property( 'ellipsize', value )
