# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import CurrentSong
import sys
import gettext
import os
import socket
import urllib, urllib2


def _extract_fields(reply, fields):
    """Return a dict associating a value to each requested field from a MPD
    reply.

    Parameters:
        reply - MPD reply (string)
        fields - sequence of fields (lowercase) we'll associate a value to.
    
    Here's an example of an MPD reply:
    
        Artist: foo
        Title: bar
        Album: other
        OK
    """
    # Initialize dict values to empty strings in case they are not in
    # the MPD reply.
    ret = {}
    for field in fields:
        ret[field] = ""

    for line in reply.splitlines():
        if ":" in line:
            field, value = line.split(":", 1)
            field = field.strip().lower()
            if field in fields:
                ret[field] = value.strip()
    return ret

RETRYNUM = 1000

class Mpd( CurrentSong.CurrentSong ):
    
    customConfig = {
        'host': '127.0.0.1',
        'port': '6600',
    }
    
    def __init__(self):
        CurrentSong.CurrentSong.__init__(self)
        self._socket = None
        self._retNum = 0

        self.song_info = {
            'artist': '',
            'title': '',
            'album': '',
        }

    def updateConfig(self):
        if self.connected():
            self._socket.close()
            self._socket = None
        self._connect()
        for item in self.song_info.keys():
            self.song_info[item] = ""

    def tryReconnect():
        if self._retNum > RETRYNUN:
            self._retNum = 0
            self._connect()
        else:
            self._retNum += 1

    def _getSocket(self):
        if self._socket == None:
            self.tryReconnect()
        return self._socket

    def _connect(self):
        if self.connected():
            return

        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self._socket.connect((self.customConfig['host'], int(self.customConfig['port'])))
            if not self._socket.recv(128).startswith("OK"):
                raise
        except socket.error, msg:
            print "can't connect to", (self.customConfig['host'], int(self.customConfig['port']))
            self._socket.close()
            self._socket = None
        return 
    
    def connected(self):
        if self._socket is None:
            return False

        return True
    
    def start(self):
        self._connect()
        return True

    def check(self):
        if self.checkSongChanged():
            self.artist = self.getArtist()
            self.title = self.getTitle()
            self.album = self.getAlbum()
            return True
        return False
        
    def stop(self):
        '''Close connection'''
        if self.connected():
            self._socket.close()
        self._socket = None

    def setHost(self, host):
        self.host = host

    def setPort(self, port):
        self.port = port

    def getArtist(self):
        return self.song_info["artist"]
    
    def getAlbum(self):
        return self.song_info["album"]
    
    def getTitle(self):
        return self.song_info["title"]
    
    def sendCmd(self, cmd):
        try:
            sock = self._getSocket()
            sock.send("%s\n" % cmd)
            ret = sock.recv(1024)
            # Command failed
            if ret.startswith("ACK"):
                return None
            return ret
        except:
            return None
        
    def isPlaying(self):
        status = self.sendCmd("status")
        if status is not None:
            if _extract_fields(status, ["state"])["state"] == "play":
                return True

        return False

    def checkSongChanged(self):
        if not self.isPlaying():
            if "".join(self.song_info.values()) != "":
                # MPD stopped playing, so empty all fields
                for item in self.song_info.keys():
                    self.song_info[item] = ""      
                return True
        else: 
            song_rpl = self.sendCmd("currentsong")
            if song_rpl is not None:
                song_info = _extract_fields(song_rpl, self.song_info.keys())
                if self.song_info != song_info:
                    self.song_info = song_info
                    return True
            
        return False

    def search_local_image(self,artist,album):
        # return False if no local image found
        file_dsc = os.path.expanduser("~/.covers/" + artist + "-" + album + ".jpg")     
    
        if os.path.exists(file_dsc):
            return file_dsc
        
        return None

    def search_remote_image(self, artist, album ):
        # Returns None if no images found
        imgfound = None
        if len(artist) == 0 or len(album) == 0:
            return imgfound
        dest_filename = os.path.expanduser("~/.covers/" + artist + "-" + album + ".jpg") 
        try:
            artist = urllib.quote(artist)
            album = urllib.quote(album)
            amazon_key = "12DR2PGAQT303YTEWP02"
            search_url = "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazon_key + "&Operation=ItemSearch&SearchIndex=Music&Artist=" + artist + "&ResponseGroup=Images&Keywords=" + album
            request = urllib2.Request(search_url)
            opener = urllib2.build_opener()
            f = opener.open(request).read()
            curr_pos = 200    # Skip header..
            # Check if any results were returned; if not, search
            # again with just the artist name:
            img_url = f[f.find("<URL>http://", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
            if len(img_url) == 0:
                search_url = "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazon_key + "&Operation=ItemSearch&SearchIndex=Music&Artist=" + artist + "&ResponseGroup=Images"
                request = urllib2.Request(search_url)
                opener = urllib2.build_opener()
                f = opener.open(request).read()
                img_url = f[f.find("<URL>http://", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
                # And if that fails, try one last time with just the album name:
                if len(img_url) == 0:
                   search_url = "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazon_key + "&Operation=ItemSearch&SearchIndex=Music&ResponseGroup=Images&Keywords=" + album
                   request = urllib2.Request(search_url)
                   request.add_header('Accept-encoding', 'gzip')
                   opener = urllib2.build_opener()
                   f = opener.open(request).read()
                   img_url = f[f.find("<URL>http://", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
     
            curr_pos = f.find("<LargeImage>", curr_pos+10)
            img_url = f[f.find("<URL>http://", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
            if len(img_url) > 0:
                urllib.urlretrieve(img_url, dest_filename)
                imgfound = dest_filename
        except:
            pass
        return imgfound
    
    def getCoverPath( self ):
        file = self.search_local_image(self.artist, self.album )
        if file != None:
            return file
        else:
            return self.search_remote_image(self.artist, self.album )
